@@ -474,18 +474,16 @@ class TypesTest(unittest.TestCase):
             TPM2_SPEC.parse("foo")
 
     def test_TPM2_GENERATED_VALUE(self):
-        self.assertEqual(
-            TPM2_GENERATED_VALUE.parse("value"), TPM2_GENERATED_VALUE.VALUE
-        )
+        self.assertEqual(TPM2_GENERATED.parse("value"), TPM2_GENERATED.VALUE)
 
         with self.assertRaises(ValueError):
-            TPM2_GENERATED_VALUE.parse("")
+            TPM2_GENERATED.parse("")
 
         with self.assertRaises(TypeError):
-            TPM2_GENERATED_VALUE.parse(None)
+            TPM2_GENERATED.parse(None)
 
         with self.assertRaises(ValueError):
-            TPM2_GENERATED_VALUE.parse("foo")
+            TPM2_GENERATED.parse("foo")
 
     def test_TPM2_RC(self):
         self.assertEqual(TPM2_RC.parse("Success"), TPM2_RC.SUCCESS)
@@ -641,6 +639,12 @@ class TypesTest(unittest.TestCase):
         db = d.buffer
         self.assertEqual(db, b"test1")
 
+        name2b = TPM2B_NAME(b"\x00\x04" + b"\xAA" * 20)
+        name, offset = TPMT_HA.unmarshal(name2b)
+        self.assertEqual(offset, len(name2b))
+        self.assertEqual(name.hashAlg, TPM2_ALG.SHA1)
+        self.assertEqual(bytes(name.digest.sha1), b"\xAA" * 20)
+
     def test_unsupported_unmarshal(self):
         with self.assertRaises(RuntimeError) as e:
             TPM_OBJECT.unmarshal(b"")
@@ -1040,6 +1044,23 @@ class TypesTest(unittest.TestCase):
 
         self.assertEqual(templ.parameters.asymDetail.symmetric.algorithm, TPM2_ALG.NULL)
 
+    def test_TPMT_PUBLIC_parse_ecc_sm2(self):
+
+        templ = TPMT_PUBLIC.parse(alg="ecc_sm2")
+
+        self.assertEqual(templ.nameAlg, TPM2_ALG.SHA256)
+
+        self.assertEqual(
+            templ.objectAttributes, TPMA_OBJECT.DEFAULT_TPM2_TOOLS_CREATE_ATTRS
+        )
+
+        self.assertEqual(templ.type, TPM2_ALG.ECC)
+        self.assertEqual(templ.parameters.eccDetail.curveID, TPM2_ECC_CURVE.SM2_P256)
+
+        self.assertEqual(templ.parameters.eccDetail.scheme.scheme, TPM2_ALG.NULL)
+
+        self.assertEqual(templ.parameters.asymDetail.symmetric.algorithm, TPM2_ALG.NULL)
+
     def test_TPMT_PUBLIC_parse_ecc_camellia(self):
         templ = TPMT_PUBLIC.parse(alg="ecc:camellia128cfb")
         self.assertEqual(
@@ -1050,6 +1071,14 @@ class TypesTest(unittest.TestCase):
             templ.parameters.eccDetail.symmetric.mode.camellia, TPM2_ALG.CFB
         )
 
+    def test_TPMT_PUBLIC_parse_ecc_sm4(self):
+        templ = TPMT_PUBLIC.parse(alg="ecc:sm4128cfb")
+        self.assertEqual(templ.parameters.eccDetail.symmetric.algorithm, TPM2_ALG.SM4)
+        self.assertEqual(templ.parameters.eccDetail.symmetric.keyBits.camellia, 128)
+        self.assertEqual(
+            templ.parameters.eccDetail.symmetric.mode.camellia, TPM2_ALG.CFB
+        )
+
     def test_TPMT_PUBLIC_parse_rsa_oaep(self):
         templ = TPMT_PUBLIC.parse(
             "rsa2048:oaep-sha512",
@@ -1078,6 +1107,17 @@ class TypesTest(unittest.TestCase):
         self.assertEqual(templ.parameters.symDetail.sym.keyBits.sym, 256)
         self.assertEqual(templ.parameters.symDetail.sym.mode.sym, TPM2_ALG.CFB)
 
+    def test_TPMT_PUBLIC_parse_sm4(self):
+        templ = TPMT_PUBLIC.parse("sm4128cfb")
+        self.assertEqual(templ.type, TPM2_ALG.SYMCIPHER)
+        self.assertEqual(templ.parameters.symDetail.sym.algorithm, TPM2_ALG.SM4)
+        self.assertEqual(templ.parameters.symDetail.sym.keyBits.sym, 128)
+        self.assertEqual(templ.parameters.symDetail.sym.mode.sym, TPM2_ALG.CFB)
+
+        with self.assertRaises(ValueError) as e:
+            TPMT_PUBLIC.parse("sm4256cfb")
+        self.assertEqual(str(e.exception), 'Expected bits to be 128, got: "256"')
+
     def test_TPML_ALG_parse_none(self):
         with self.assertRaises(ValueError):
             TPML_ALG.parse(None)
@@ -1452,6 +1492,14 @@ class TypesTest(unittest.TestCase):
         with self.assertRaises(ValueError):
             TPMA_LOCALITY.create_extended(255 - 32 + 1)
 
+        loc = TPMA_LOCALITY.parse("zero|four")
+        self.assertEqual(loc, TPMA_LOCALITY.ZERO | TPMA_LOCALITY.FOUR)
+        self.assertEqual(str(loc), "zero|four")
+
+        loc = TPMA_LOCALITY.parse("240")
+        self.assertEqual(loc, 240)
+        self.assertEqual(str(loc), "0xf0")
+
     def test_TPMS_CONTEXT_from_tools(self):
         test_ctx = b"""utzA3gAAAAFAAAABgAAAAAAAAAAAAAOkAvIAAAAAAqIAIFNJEhgwU8zxMhuTBhSqPktXguCbMgUg
         mACnGHIlDr0mAn7QtSMsTy1hAOqPvR8LRxcCphVs1owzQuHIe1Ez4kwA5xSl2zU+xFMhuD9coN4Z
@@ -1468,6 +1516,15 @@ class TypesTest(unittest.TestCase):
         r+ShbNtNnOUGnAYOhak3DMmOKdpBgAAAgAAA/wAWAAR53X6WF4cq4euj360A2EG67P+iZgAAAAEA
         JgAlAAQAAwByAAAABgCAABAAFIrV77jm23RSbSe0b3NqvEBvuN35"""
 
+        test_session = b"""utzA3gAAAAIDAAu63MDeAAAAAUAAAAcDAAAAAAAAAAAAAp8BkwAAAAABGwAg3+5wcQDG7hmg8aaC
+        g94E+NcznuqznZiXkmGiTHEL9AMA99ZtPnyRs1VxIGHVZcJJJyTyYmRO8bZAyXPPwq4fy8kkwqwv
+        0PE+S0UaVvmLA31rM5WhDryFIGp4CsCD+omQdwKNRhOHYLSmKLnvnIaA/ARauSFfl76+fpSTQRsG
+        OtHQO19livPgPBCXQhfUxKWVCN0N9Wf0QXSIzj9djnk5/hQPDZ/GnoRkq22gZ8QgViS5xZ2U5kN3
+        kF8pKLv5onlTlT3NleY6G3BWBwZ6oERS3WSkaquDO5H/soA4YOx7y4UuLpIz/hgsFrf9j7vyGYcY
+        JkkNOi6i0+hWpmDFnhn0gwZL6h+dJNd0qJIv5R7bjcnW7b4sJ5gaQPwAAAMAAAAABAMAAAAAAAAD
+        AAAAAAAAABAACwAAAwEAIMn6hieIgUZusbUvHkdT+fB6m7Nor7v/p4MunwkyviTNACCFA6yl1FMW
+        Oi6NOltEBFNSgQDnMCUgvgxixgL0nVUlcwAAAAAAAAAAAAAAAAAAAAA="""
+
         ctxbytes = b64decode(test_ctx)
         ctx = TPMS_CONTEXT.from_tools(ctxbytes)
 
@@ -1486,6 +1543,13 @@ class TypesTest(unittest.TestCase):
         with self.assertRaises(ValueError):
             ctx = TPMS_CONTEXT.from_tools(badversion)
 
+        sessbytes = b64decode(test_session)
+        sess = TPMS_CONTEXT.from_tools(sessbytes)
+        self.assertEqual(sess.hierarchy, TPM2_RH.NULL)
+        self.assertEqual(sess.savedHandle, 0x03000000)
+        self.assertEqual(sess.sequence, 671)
+        self.assertEqual(sess.contextBlob, sessbytes[37:])
+
     def test_TPM_FRIENDLY_INT_str(self):
         alg = TPM2_ALG(TPM2_ALG.ECC)
         self.assertEqual(str(alg), "ecc")
@@ -1775,6 +1839,16 @@ class TypesTest(unittest.TestCase):
         ccs = str(TPMA_CC.NV | TPMA_CC.V)
         self.assertEqual(ccs, "nv|v")
 
+    def test_TPMA_SESSION(self):
+
+        x = TPMA_SESSION.CONTINUESESSION | TPMA_SESSION.DECRYPT
+        y = str(x)
+        self.assertEqual(y, "continuesession|decrypt")
+
+        x = TPMA_SESSION.CONTINUESESSION | TPMA_SESSION.DECRYPT | 0x128
+        with self.assertRaises(ValueError):
+            str(x)
+
 
 if __name__ == "__main__":
     unittest.main()
